From 0528da353fbab51a1adf93d65947999a04c9e810 Mon Sep 17 00:00:00 2001
From: =?UTF-8?q?Hugo=20Beauz=C3=A9e-Luyssen?= <beauze.h@gmail.com>
Date: Sun, 22 Jan 2012 18:24:52 +0100
Subject: [PATCH 10/11] bluray: Store subpicture/regions in an external
 structure, to be shared with the vout.

---
 modules/access/bluray.c |  174 +++++++++++++++++++++++++---------------------
 1 files changed, 95 insertions(+), 79 deletions(-)

diff --git a/modules/access/bluray.c b/modules/access/bluray.c
index 8dd01a9..874307b 100644
--- a/modules/access/bluray.c
+++ b/modules/access/bluray.c
@@ -77,33 +77,37 @@ typedef enum    OverlayStatus
     Outdated    //used to update the overlay after it has been sent to the vout
 }               OverlayStatus;
 
-struct demux_sys_t
+typedef struct bluray_overlay_t
 {
-    BLURAY         *bluray;
+    vlc_mutex_t             lock;
+    subpicture_t            *p_pic;
+    OverlayStatus           status;
+    subpicture_region_t     *p_regions;
+}   bluray_overlay_t;
+
+struct  demux_sys_t
+{
+    BLURAY              *bluray;
 
     /* Titles */
-    unsigned int    i_title;
-    unsigned int    i_longest_title;
-    input_title_t **pp_title;
+    unsigned int        i_title;
+    unsigned int        i_longest_title;
+    input_title_t       **pp_title;
 
     /* Overlay stuff */
-    vlc_mutex_t             overlay_lock;
-    subpicture_t            *p_pic[MAX_OVERLAY];
-    OverlayStatus           overlay_status[MAX_OVERLAY];
-    subpicture_region_t     *p_regions[MAX_OVERLAY];
-    int             current_overlay; // -1 if no current overlay;
+    bluray_overlay_t    overlay[MAX_OVERLAY];
+    int                 current_overlay; // -1 if no current overlay;
 
-    input_thread_t  *p_input;
-    vout_thread_t   *p_vout;
+    input_thread_t      *p_input;
+    vout_thread_t       *p_vout;
 
     /* TS stream */
-    stream_t       *p_parser;
+    stream_t            *p_parser;
 };
 
 struct subpicture_updater_sys_t
 {
-    demux_sys_t *p_demux_sys;
-    int         i_plane;
+    bluray_overlay_t    *p_overlay;
 };
 
 /*****************************************************************************
@@ -152,7 +156,8 @@ static int blurayOpen( vlc_object_t *object )
     {
         return VLC_ENOMEM;
     }
-    vlc_mutex_init( &p_sys->overlay_lock );
+    for ( int i = 0; i < MAX_OVERLAY; i++ )
+        vlc_mutex_init( &p_sys->overlay[i].lock );
     p_sys->current_overlay = -1;
 
     /* init demux info fields */
@@ -296,7 +301,8 @@ static void blurayClose( vlc_object_t *object )
         vlc_object_release( p_sys->p_input );
     if (p_sys->p_parser)
         stream_Delete(p_sys->p_parser);
-    vlc_mutex_destroy( &p_sys->overlay_lock );
+    for ( int i = 0; i < MAX_OVERLAY; ++i )
+    vlc_mutex_destroy( &p_sys->overlay[i].lock );
     /* Titles */
     for (unsigned int i = 0; i < p_sys->i_title; i++)
         vlc_input_title_Delete(p_sys->pp_title[i]);
@@ -318,11 +324,11 @@ static int      subpictureUpdaterValidate( subpicture_t *p_subpic,
     VLC_UNUSED( p_fmt_src ); VLC_UNUSED( p_fmt_dst );
     VLC_UNUSED( i_ts );
     subpicture_updater_sys_t    *p_upd_sys = p_subpic->updater.p_sys;
-    demux_sys_t                 *p_sys = p_upd_sys->p_demux_sys;
+    bluray_overlay_t            *p_overlay = p_upd_sys->p_overlay;
 
-    vlc_mutex_lock( &p_sys->overlay_lock );
-    int res = p_sys->overlay_status[p_upd_sys->i_plane] == Outdated;
-    vlc_mutex_unlock( &p_sys->overlay_lock );
+    vlc_mutex_lock( &p_overlay->lock );
+    int res = p_overlay->status == Outdated;
+    vlc_mutex_unlock( &p_overlay->lock );
     return res;
 }
 
@@ -363,15 +369,15 @@ static void     subpictureUpdaterUpdate( subpicture_t *p_subpic,
 {
     VLC_UNUSED( p_fmt_src ); VLC_UNUSED( p_fmt_dst ); VLC_UNUSED( i_ts );
     subpicture_updater_sys_t    *p_upd_sys = p_subpic->updater.p_sys;
-    demux_sys_t                 *p_sys = p_upd_sys->p_demux_sys;
+    bluray_overlay_t            *p_overlay = p_upd_sys->p_overlay;
 
     /*
      * When this function is called, all p_subpic regions are gone.
      * We need to duplicate our regions (stored internaly) to this subpic.
      */
-    vlc_mutex_lock( &p_sys->overlay_lock );
-    p_subpic->p_region = subpicture_region_Clone( p_sys->p_regions[p_upd_sys->i_plane] );
-    subpicture_region_t *p_src = p_sys->p_regions[p_upd_sys->i_plane]->p_next;
+    vlc_mutex_lock( &p_overlay->lock );
+    p_subpic->p_region = subpicture_region_Clone( p_overlay->p_regions );
+    subpicture_region_t *p_src = p_overlay->p_regions->p_next;
     subpicture_region_t *p_dst = p_subpic->p_region;
     while ( p_src != NULL )
     {
@@ -382,8 +388,8 @@ static void     subpictureUpdaterUpdate( subpicture_t *p_subpic,
         p_src = p_src->p_next;
     }
     p_dst->p_next = NULL;
-    p_sys->overlay_status[p_upd_sys->i_plane] = Displayed;
-    vlc_mutex_unlock( &p_sys->overlay_lock );
+    p_overlay->status = Displayed;
+    vlc_mutex_unlock( &p_overlay->lock );
 }
 
 static void     subpictureUpdaterDestroy( subpicture_t *p_subpic )
@@ -399,26 +405,26 @@ static void blurayDeleteOverlaySubPic( demux_t *p_demux, int i_plane )
 {
     demux_sys_t     *p_sys = p_demux->p_sys;
 
-    vlc_mutex_lock( &p_sys->overlay_lock );
-    p_sys->overlay_status[i_plane] = Closed;
-    if ( p_sys->p_pic[i_plane] == NULL )
+    vlc_mutex_lock( &p_sys->overlay[i_plane].lock );
+    p_sys->overlay[i_plane].status = Closed;
+    if ( p_sys->overlay[i_plane].p_pic == NULL )
     {
-        vlc_mutex_unlock( &p_sys->overlay_lock );
+        vlc_mutex_unlock( &p_sys->overlay[i_plane].lock );
         return ;
     }
     //If we have a vout, let it cleanup by itself.
     if ( p_sys->p_vout )
-        vout_FlushSubpictureChannel( p_sys->p_vout, p_sys->p_pic[i_plane]->i_channel );
+        vout_FlushSubpictureChannel( p_sys->p_vout, p_sys->overlay[i_plane].p_pic->i_channel );
     else
     {
         /*
          * Else, the subpicture hasn't been sent yet, clean it ourselves.
          * In that case, the vout doesn't hold the picture, we can use it safely
          */
-        subpicture_Delete( p_sys->p_pic[i_plane] );
+        subpicture_Delete( p_sys->overlay[i_plane].p_pic );
     }
-    p_sys->p_pic[i_plane] = NULL;
-    vlc_mutex_unlock( &p_sys->overlay_lock );
+    p_sys->overlay[i_plane].p_pic = NULL;
+    vlc_mutex_unlock( &p_sys->overlay[i_plane].lock );
 }
 
 static void blurayCloseOverlay( demux_t *p_demux )
@@ -445,13 +451,13 @@ static void blurayActivateOverlay( demux_t *p_demux, const BD_OVERLAY* const ov
      * If the overlay is already displayed, mark the picture as outdated.
      * We must NOT use vout_PutSubpicture if a picture is already displayed.
      */
-    vlc_mutex_lock( &p_sys->overlay_lock );
-    if ( ( p_sys->overlay_status[ov->plane] == Displayed ||
-           p_sys->overlay_status[ov->plane] == Outdated )
+    vlc_mutex_lock( &p_sys->overlay[ov->plane].lock );
+    if ( ( p_sys->overlay[ov->plane].status == Displayed ||
+           p_sys->overlay[ov->plane].status == Outdated )
          && p_sys->p_vout )
     {
-        p_sys->overlay_status[ov->plane] = Outdated;
-        vlc_mutex_unlock( &p_sys->overlay_lock );
+        p_sys->overlay[ov->plane].status = Outdated;
+        vlc_mutex_unlock( &p_sys->overlay[ov->plane].lock );
         return ;
     }
     /*
@@ -460,22 +466,21 @@ static void blurayActivateOverlay( demux_t *p_demux, const BD_OVERLAY* const ov
      * the overlay is computed
      */
     p_sys->current_overlay = ov->plane;
-    p_sys->overlay_status[ov->plane] = ToDisplay;
-    vlc_mutex_unlock( &p_sys->overlay_lock );
+    p_sys->overlay[ov->plane].status = ToDisplay;
+    vlc_mutex_unlock( &p_sys->overlay[ov->plane].lock );
 }
 
 static void blurayInitOverlay( demux_t *p_demux, const BD_OVERLAY* const ov )
 {
     demux_sys_t     *p_sys = p_demux->p_sys;
 
-    if ( p_sys->p_pic[ov->plane] != NULL )
+    if ( p_sys->overlay[ov->plane].p_pic != NULL )
         blurayDeleteOverlaySubPic( p_demux, ov->plane );
 
     subpicture_updater_sys_t    *p_upd_sys = malloc( sizeof( *p_upd_sys ) );
     if ( unlikely( p_upd_sys == NULL ) )
         return ;
-    p_upd_sys->p_demux_sys = p_sys;
-    p_upd_sys->i_plane = ov->plane;
+    p_upd_sys->p_overlay = &p_sys->overlay[ov->plane];
     subpicture_updater_t    updater =
     {
         .pf_validate = subpictureUpdaterValidate,
@@ -483,22 +488,22 @@ static void blurayInitOverlay( demux_t *p_demux, const BD_OVERLAY* const ov )
         .pf_destroy = subpictureUpdaterDestroy,
         .p_sys = p_upd_sys,
     };
-    p_sys->p_pic[ov->plane] = subpicture_New( &updater );
-    p_sys->p_pic[ov->plane]->i_original_picture_width = ov->w;
-    p_sys->p_pic[ov->plane]->i_original_picture_height = ov->h;
-    p_sys->p_pic[ov->plane]->b_ephemer = true;
-    p_sys->p_pic[ov->plane]->b_absolute = true;
+    p_sys->overlay[ov->plane].p_pic = subpicture_New( &updater );
+    p_sys->overlay[ov->plane].p_pic->i_original_picture_width = ov->w;
+    p_sys->overlay[ov->plane].p_pic->i_original_picture_height = ov->h;
+    p_sys->overlay[ov->plane].p_pic->b_ephemer = true;
+    p_sys->overlay[ov->plane].p_pic->b_absolute = true;
 }
 
 static void blurayClearOverlay( demux_t *p_demux, const BD_OVERLAY* const ov )
 {
     demux_sys_t     *p_sys = p_demux->p_sys;
 
-    vlc_mutex_lock( &p_sys->overlay_lock );
-    subpicture_region_ChainDelete( p_sys->p_regions[ov->plane] );
-    p_sys->p_regions[ov->plane] = NULL;
-    p_sys->overlay_status[ov->plane] = Outdated;
-    vlc_mutex_unlock( &p_sys->overlay_lock );
+    vlc_mutex_lock( &p_sys->overlay[ov->plane].lock );
+    subpicture_region_ChainDelete( p_sys->overlay[ov->plane].p_regions );
+    p_sys->overlay[ov->plane].p_regions = NULL;
+    p_sys->overlay[ov->plane].status = Outdated;
+    vlc_mutex_unlock( &p_sys->overlay[ov->plane].lock );
 }
 
 static void blurayDrawOverlay( demux_t *p_demux, const BD_OVERLAY* const ov )
@@ -511,7 +516,7 @@ static void blurayDrawOverlay( demux_t *p_demux, const BD_OVERLAY* const ov )
      */
     if ( ov->img )
     {
-        vlc_mutex_lock( &p_sys->overlay_lock );
+        vlc_mutex_lock( &p_sys->overlay[ov->plane].lock );
 
         video_format_t          fmt;
         video_format_Init( &fmt, 0 );
@@ -546,11 +551,11 @@ static void blurayDrawOverlay( demux_t *p_demux, const BD_OVERLAY* const ov )
         }
         //Append the new region at the end of our current regions:
         //If there's no region yet, don't bother checking for update...
-        subpicture_region_t *p_reg = p_sys->p_regions[ov->plane];
+        subpicture_region_t *p_reg = p_sys->overlay[ov->plane].p_regions;
         if ( p_reg == NULL )
         {
-            p_sys->p_regions[ov->plane] = region;
-            vlc_mutex_unlock( &p_sys->overlay_lock );
+            p_sys->overlay[ov->plane].p_regions = region;
+            vlc_mutex_unlock( &p_sys->overlay[ov->plane].lock );
             return ;
         }
         /*
@@ -566,9 +571,9 @@ static void blurayDrawOverlay( demux_t *p_demux, const BD_OVERLAY* const ov )
             {
                 if ( p_prev == NULL )
                 {
-                    p_sys->p_regions[ov->plane] = p_reg->p_next;
+                    p_sys->overlay[ov->plane].p_regions = p_reg->p_next;
                     subpicture_region_Delete( p_reg );
-                    p_reg = p_sys->p_regions[ov->plane];
+                    p_reg = p_sys->overlay[ov->plane].p_regions;
                 }
                 else
                 {
@@ -585,10 +590,10 @@ static void blurayDrawOverlay( demux_t *p_demux, const BD_OVERLAY* const ov )
         }
         /* If there's only one region, both prev and reg will be NULL */
         if ( p_prev == NULL )
-            p_sys->p_regions[ov->plane] = region;
+            p_sys->overlay[ov->plane].p_regions = region;
         else
             p_prev->p_next = region;
-        vlc_mutex_unlock( &p_sys->overlay_lock );
+        vlc_mutex_unlock( &p_sys->overlay[ov->plane].lock );
     }
     /*
      * /!\ The region is now stored in our internal list, but not in the subpicture /!\
@@ -659,13 +664,22 @@ static void bluraySendOverlayToVout( demux_t *p_demux )
     demux_sys_t *p_sys = p_demux->p_sys;
 
     //Sending subpicture to vout:
-    p_sys->p_pic[p_sys->current_overlay]->i_start =
-            p_sys->p_pic[p_sys->current_overlay]->i_stop = mdate();
-    p_sys->p_pic[p_sys->current_overlay]->i_channel =
+    p_sys->overlay[p_sys->current_overlay].p_pic->i_start =
+            p_sys->overlay[p_sys->current_overlay].p_pic->i_stop = mdate();
+    p_sys->overlay[p_sys->current_overlay].p_pic->i_channel =
             vout_RegisterSubpictureChannel( p_sys->p_vout );
-    vout_PutSubpicture( p_sys->p_vout, p_sys->p_pic[p_sys->current_overlay] );
-    //Mark the picture as updated, as it contains no region for now.
-    p_sys->overlay_status[p_sys->current_overlay] = Outdated;
+    /*
+     * Sending the subpicture to vout.
+     * This must be done only once per subpicture, ie. only once between each
+     * blurayInitOverlay & blurayCloseOverlay call.
+     */
+    vout_PutSubpicture( p_sys->p_vout, p_sys->overlay[p_sys->current_overlay].p_pic );
+
+    /*
+     * Mark the picture as updated, as it contains no region for now.
+     * This will make the subpicture_updater_t call pf_update
+     */
+    p_sys->overlay[p_sys->current_overlay].status = Outdated;
     //Activating vout events handling:
     var_AddCallback( p_sys->p_vout, "mouse-moved", &onMouseEvent, p_demux );
     var_AddCallback( p_sys->p_vout, "mouse-clicked", &onMouseEvent, p_demux );
@@ -982,18 +996,20 @@ static int blurayDemuxMenu( demux_t *p_demux )
     }
     p_block->i_buffer = nread;
 
-    vlc_mutex_lock( &p_sys->overlay_lock );
-    if ( p_sys->current_overlay != -1 &&
-         p_sys->overlay_status[p_sys->current_overlay] == ToDisplay )
+    if ( p_sys->current_overlay != -1 )
     {
-        vlc_mutex_unlock( &p_sys->overlay_lock );
-        if ( p_sys->p_vout == NULL )
-            p_sys->p_vout = input_GetVout( p_sys->p_input );
-        if ( p_sys->p_vout != NULL )
-            bluraySendOverlayToVout( p_demux );
+        vlc_mutex_lock( &p_sys->overlay[p_sys->current_overlay].lock );
+        if ( p_sys->overlay[p_sys->current_overlay].status == ToDisplay )
+        {
+            vlc_mutex_unlock( &p_sys->overlay[p_sys->current_overlay].lock );
+            if ( p_sys->p_vout == NULL )
+                p_sys->p_vout = input_GetVout( p_sys->p_input );
+            if ( p_sys->p_vout != NULL )
+                bluraySendOverlayToVout( p_demux );
+        }
+        else
+            vlc_mutex_unlock( &p_sys->overlay[p_sys->current_overlay].lock );
     }
-    else
-        vlc_mutex_unlock( &p_sys->overlay_lock );
     stream_DemuxSend( p_sys->p_parser, p_block );
     return 1;
 }
-- 
1.7.8.4

